/*
 * Access a server offering a compound interface.
 *
 * Copyright (C) 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>

#include <stdio.h>

#include "calc_client.h"
#include "counter_client.h"

int main(void)
{
  l4_cap_idx_t server = l4re_env_get_cap("server");

  /* Reference to a calculator and encapsulation. */

  ref_Calc calc_ref = {.cap=server};
  Calc calc_obj = {.ref=calc_ref, .iface=&client_iface_Calc};

  /* Reference to a counter and encapsulation. */

  ref_Counter counter_ref = {.cap=server};
  Counter counter_obj = {.ref=counter_ref, .iface=&client_iface_Counter};

  /* Call the calculator. */

  int result;

  /* Invoke the operation function directly. */

  Calc_add(calc_ref, 123, 456, &result);
  printf("add 123, 456 -> %d\n", result);

  /* Invoke the operation via the interface. */

  calc_obj.iface->add(calc_obj.ref, 789, 123, &result);
  printf("add 789, 123 -> %d\n", result);

  /* Call the counter. */

  /* Invoke the operation function directly. */

  Counter_increment(counter_ref, &result);
  printf("increment -> %d\n", result);

  /* Invoke the operation via the interface. */

  counter_obj.iface->increment(counter_obj.ref, &result);
  printf("increment -> %d\n", result);

  return 0;
}
